/*
 * Include some standard vectrex functions first!
 */
#include <vectrex.h>
#include <ymsound.h>

/* NON CONSTANT VARIABLES NOT ALLOWED TO BE INITIALIZED */

/*
 * Some defines, Maximal brightness is $7f, highest not set!
 * Max Scale would be $ff, well here we take only $f0!
 */
#define MAX_BRIGHTNESS (0x7f)
#define LOW_BRIGHTNESS (0x2f)

#define NO_OF_SONGS 9
extern const void pac4_data;
extern const char pac4_name;

extern const void RAINBOW_data;
extern const char RAINBOW_name;

extern const void BEBOP_data;
extern const char BEBOP_name;

extern const void RAINBOW_data;
extern const char RAINBOW_name;

extern const void STARWARS_data;
extern const char STARWARS_name;

extern const void TITLE_data;
extern const char TITLE_name;

extern const void AXELF_data;
extern const char AXELF_name;

extern const void DOH3_data;
extern const char DOH3_name;

extern const void COMNDO2_data;
extern const char COMNDO2_name;

extern const void MM_data;
extern const char MM_name;

/*
 if non volatile compiler error!

 static unsigned char chosen;
 static volatile unsigned char chosen;
*/
/* slsl slsl look at asm *.ss */
static unsigned char chosen;

struct song
{
 int x;
 int y;
 const char *name;
 const void *data;
};

struct song songs[NO_OF_SONGS];

/*
 * Inlined static... insures this is 'really' inlined completely...
 *
 * oops... a small one :-)
 *
 */
static inline void start_one_vectrex_round(void)
{
  ym_sound();
  if (ym_cdata == 0)
  {
   ym_init(songs[chosen].data); /* 'C' callable init, sound data */
  }
  joy_digital();
  wait_recal();
  intensity(MAX_BRIGHTNESS);  /* set intensity of vector beam... */
}

/*
 * Inlined static... insures this is 'really' inlined completely...
 *
 * Initialize all dots to starting defaults...
 *
 */
static inline void init(void)
{
 songs[0].x= -128;
 songs[0].y= 100;
 songs[0].name= &(pac4_name);
 songs[0].data= &(pac4_data);

 songs[1].x= -128;
 songs[1].y= 80;
 songs[1].name= &(RAINBOW_name);
 songs[1].data= &(RAINBOW_data);

 songs[2].x= -128;
 songs[2].y= 60;
 songs[2].name= &(TITLE_name);
 songs[2].data= &(TITLE_data);

 songs[3].x= -128;
 songs[3].y= 40;
 songs[3].name= &(BEBOP_name);
 songs[3].data= &(BEBOP_data);

 songs[4].x= -128;
 songs[4].y= 20;
 songs[4].name= &(STARWARS_name);
 songs[4].data= &(STARWARS_data);

 songs[5].x= -128;
 songs[5].y= 0;
 songs[5].name= &(AXELF_name);
 songs[5].data= &(AXELF_data);

 songs[6].x= -128;
 songs[6].y= -20;
 songs[6].name= &(DOH3_name);
 songs[6].data= &(DOH3_data);

 songs[7].x= -128;
 songs[7].y= -40;
 songs[7].name= &(COMNDO2_name);
 songs[7].data= &(COMNDO2_data);

 songs[8].x= -128;
 songs[8].y= -60;
 songs[8].name= &(MM_name);
 songs[8].data= &(MM_data);

 chosen = 0;
 ym_init(songs[chosen].data); /* 'C' callable init, sound data */
}

/*
 * Main ... what else to say?
 */
int main(void)
{
  unsigned char i;              /* a counter */
  char last;
  init();                       /* initialize ... */
  last=chosen;
  do
  {
   wait_recal();
   intensity(MAX_BRIGHTNESS);  /* set intensity of vector beam... */
   print_str(-128,10, "USE CURSOR TO CHOOSE SONG!");
   joy_digital();
  }
  while (!joystick1_y);
  while (true)                  /* do forever... the following */
  {
    start_one_vectrex_round();  /* start vectrex round */
    print_str(songs[chosen].x, songs[chosen].y, songs[chosen].name);
    if (joystick1_y<0) /* DOWN */
    {
     if (chosen < NO_OF_SONGS-1)
     {
      if (joystick1_y != last)
      {
       chosen++;
       ym_init(songs[chosen].data); /* 'C' callable init, sound data */
      }
     }
    }
    else if (joystick1_y>0) /* UP */
    {
     if (chosen > 0)
     {
      if (joystick1_y != last)
      {
       chosen--;
       ym_init(songs[chosen].data); /* 'C' callable init, sound data */
      }
     }
    }
    last = joystick1_y;
  }
}


/* END OF FILE */
