#include "trig.h"

// helper is used by the macros to
// have some 'tmp' space helping and optimizing so calculations
// and z -> well the never (in parallel projection) used z coordinate
signed char helper;
signed char z;

// used in macros
// the always should have the correct
// values for the cos() sin() of the angles for the axis
signed char cosx;
signed char sinx;
signed char cosy;
signed char siny;
signed char cosz;
signed char sinz;

// not used here
// but somewhere you should have the angles
// I left them here, because they were here
// from the start
// angles must be from 0 - 63 (that is 0 degrees to 360 degrees)
unsigned short angle_x;
unsigned short angle_y;
unsigned short angle_z;

// the (RAM) array of all precalculated 3d vectors
// used in the 'inits'
// all vector lists point to that array thru below defined makros
// x,y,z -> 3 possible direction is 3*3*3 = 27 and 1 for each coordinate
// that is 27 * 3!
signed char allDirs_calc[27 * 3];

// all 'vector' locations and inverse locations
// as pointers to above Array
#define _0_0_0 (&allDirs_calc[0])
#define _1_0_0 (&allDirs_calc[3])
#define _1_1_0 (&allDirs_calc[6])
#define _1_0_1 (&allDirs_calc[9])
#define _1_1_1 (&allDirs_calc[12])
#define _0_1_0 (&allDirs_calc[15])
#define _0_1_1 (&allDirs_calc[18])
#define _0_0_1 (&allDirs_calc[21])
#define _N_1_0 (&allDirs_calc[24])
#define _N_0_1 (&allDirs_calc[27])
#define _0_N_1 (&allDirs_calc[30])
#define _N_1_1 (&allDirs_calc[33])
#define _1_N_1 (&allDirs_calc[36])
#define _1_1_N (&allDirs_calc[39])

#define INVERS_OFFSET 42

#define I_0_0_0 (_0_0_0 + INVERS_OFFSET)
#define I_1_0_0 (_1_0_0 + INVERS_OFFSET)
#define I_1_1_0 (_1_1_0 + INVERS_OFFSET)
#define I_1_0_1 (_1_0_1 + INVERS_OFFSET)
#define I_1_1_1 (_1_1_1 + INVERS_OFFSET)
#define I_0_1_0 (_0_1_0 + INVERS_OFFSET)
#define I_0_1_1 (_0_1_1 + INVERS_OFFSET)
#define I_0_0_1 (_0_0_1 + INVERS_OFFSET)
#define I_N_1_0 (_N_1_0 + INVERS_OFFSET)
#define I_N_0_1 (_N_0_1 + INVERS_OFFSET)
#define I_0_N_1 (_0_N_1 + INVERS_OFFSET)
#define I_N_1_1 (_N_1_1 + INVERS_OFFSET)
#define I_1_N_1 (_1_N_1 + INVERS_OFFSET)
#define I_1_1_N (_1_1_N + INVERS_OFFSET)

#define _N_0_0 I_1_0_0
#define _N_N_0 I_1_1_0
#define _N_0_N I_1_0_1
#define _N_N_N I_1_1_1
#define _0_N_0 I_0_1_0
#define _0_N_N I_0_1_1
#define _0_0_N I_0_0_1
#define _1_N_0 I_N_1_0
#define _1_0_N I_N_0_1
#define _0_1_N I_0_N_1
#define _1_N_N I_N_1_1
#define _N_1_N I_1_N_1
#define _N_N_1 I_1_1_N

// this STA and ADDA cause the compiler to hickup
// it is because of these to instructions I 'SED' the resulting source
// these makros are used to be able to do inline assembly
// with different parameters
// look at the direction *.h files, there they are used
#define STORE_A(address) asm("sta %0"::"i" ((unsigned short)(&address)));
#define ADD_A_TO(address) asm("adda %0"::"i" ((unsigned short)(&address)));STORE_A(address);
#define SUB_A_FROM(address) asm("nega");ADD_A_TO(address);
#define STORE_A_NEG(address) asm("nega");asm("sta %0"::"i" ((unsigned short)(&address)));
#define ADD_A_TO_NEG(address) asm("adda %0"::"i" ((unsigned short)(&address)));STORE_A_NEG(address);
#define SUB_A_FROM_NEG(address) asm("nega");ADD_A_TO_NEG(address);

// this does:
// signed multiplication of A and B to D
// and divides D by 64
// result is stored in A
// I must branch to 'offsets' rather than to labels
// since this is a makro and labels would not be
// localized... gives certainly errors of redeclarations of labels...
#define A_EQUALS_MUL(a,b)  asm ("tsta"::"A" ((unsigned char)(a)),"B" ((unsigned char)(b)));\
                   asm ("bpl . + 9"); /* goto: a+ b?*/ \
                   asm ("nega");\
                   asm ("tstb");\
                   asm ("bpl . + 9"); /*goto: was a-: a+ b+ */\
                   asm ("negb");\
                   asm ("bpl . + 11"); /* goto: was a- b-: a+ b+ */\
                   asm ("tstb"); /* a+ b?*/ \
                   asm ("bpl . + 8"); /* goto: a+ b+ */ \
                   asm ("negb"); /* here: was b-: a+ b+ */ \
                   asm ("mul"); /* was a-: a+ b+ */ \
                   asm ("comb"); /* here we can use this as negd */\
                   asm ("coma"); /* since the low nibble of b doesn't interest us */ \
                   asm ("bra . + 3");\
                   asm ("mul");  /* a+ b+ & was a- b-: a+ b+ */ \
                   asm ("aslb"); /* this divides d by 64 */ \
                   asm ("rola");\
                   asm ("aslb");\
                   asm ("rola");

// pattern, strength, point
// pattern:   255 -> draw full
//          2-254 -> draw pattern
//              0 -> move only
//              1 -> stop

// strength = scale
// scale only supported in 'draw_3ds'
struct Line3d
{
 signed char pattern;
 signed char strength;
 signed char *moveTo;
};

// the direction *.h files
// using following specials gains many cycles for 3d calculating!
// boring to create, but there are only 13 'real' ones
// and the resulting gain in cycles was significant
// so - boring but worth it
#include "000.h"
#include "100.h"
#include "110.h"
#include "101.h"
#include "111.h"
#include "010.h"
#include "011.h"
#include "001.h"
#include "n10.h"
#include "n01.h"
#include "0n1.h"
#include "n11.h"
#include "1n1.h"
#include "11n.h"

// oh well, since we use an own format... we use an own
// drawing routine
// actually it is more copy paste from the BIOS
// than anything else
// only one redirection added
// (and a slight optimization)
void draw_3d(struct Line3d *figure)
{
 save_frame_x();
 asm("       ldu 2,x"::"x" ((unsigned char*)(figure))); // take figure and put it to X
 asm("start: ldd ,u");
 asm("       sta *1");
 asm("       clr *0");
 asm("       lda ,x");
 asm("       inc *0");
 asm("       stb *1");
 asm("       sta *0H0A");
 asm("       clr *5");
 asm("       leax 4,x");
 asm("       ldu 2,x");
 asm("       lda ,x");
 asm("       bgt end1");
 asm("       ldd #0H0040");
 asm("wait:  bitb *0H0D");
 asm("       beq wait");
 asm("       sta *0H0A");
 asm("       bra start");
 asm("end1:  ldd #0H0040");
 asm("end:   bitb *0H0D");
 asm("       beq end");
 asm("       sta *0H0A":::"a","b","d","x","u"); // mark registers as changed
 restore_frame_x();
}

// apart from using strength... same as above
void draw_3ds(struct Line3d *figure)
{
 save_frame_x();
 asm("       ldu 2,x"::"x" ((unsigned char*)(figure)));
 asm("       lda 1,x");
 asm("starts:sta *4");
 asm("       ldd ,u");
 asm("       sta *1");
 asm("       clr *0");
 asm("       lda ,x");
 asm("       inc *0");
 asm("       stb *1");
 asm("       sta *0H0A");
 asm("       clr *5");
 asm("       leax 4,x");
 asm("       ldu 2,x");
 asm("       lda ,x");
 asm("       bgt end1s");
 asm("       lda 1,x");
 asm("       ldb #0H40");
 asm("waits: bitb *0H0D");
 asm("       beq waits");
 asm("       clr *0H0A");
 asm("       bra starts");
 asm("end1s: ldd #0H0040");
 asm("ends:  bitb *0H0D");
 asm("       beq ends");
 asm("       sta *0H0A":::"a","b","d","x","u");
 restore_frame_x();
}


// OLD
// following stuff is not used anymore
// and is commented out
//
// but the information herein may come handy
// this was done to test the 3d stuff
// now that everything works fine
// we use the more optimized and less well readable
// makros...

// the old stuff was used before all the makros were
// done.
// the CALC_XYZ(x,y,z) makro
// calculates the x,y,z according to the angles
// as you can see there are many if clauses in there
// doing it all directly spares us much 'time'

// these are the 'fixed' supported directions, only needed when doing it
// the old way
/*
static signed char allDirs[] =
{
        // only positive
         0, 0, 0,
         1, 0, 0,
         1, 1, 0,
         1, 0, 1,
         1, 1, 1,
         0, 1, 0,
         0, 1, 1,
         0, 0, 1,

        -1, 1, 0,
        -1, 0, 1,
         0,-1, 1,

        -1, 1, 1,
         1,-1, 1,
         1, 1,-1,

// repeated invers!, not used actually
         0, 0, 0,
        -1, 0, 0,
        -1,-1, 0,
        -1, 0,-1,
        -1,-1,-1,
         0,-1, 0,
         0,-1,-1,
         0, 0,-1,

         1,-1, 0,
         1, 0,-1,
         0, 1,-1,

         1,-1,-1,
        -1, 1,-1,
        -1,-1, 1,
};

#define STORE_A_X asm("sta _x");
#define STORE_A_Y asm("sta _y");
#define STORE_A_Z asm("sta _z");
#define ADD_A_TO_X asm("adda _x");STORE_A_X;
#define ADD_A_TO_Y asm("adda _y");STORE_A_Y;
#define ADD_A_TO_Z asm("adda _z");STORE_A_Z;
#define SUB_A_FROM_X asm("nega");ADD_A_TO_X;
#define SUB_A_FROM_Y asm("nega");ADD_A_TO_Y;
#define SUB_A_FROM_Z asm("nega");ADD_A_TO_Z;

#define MUL_B(a,b)  asm ("tsta"::"A" ((unsigned char)(a)),"B" ((unsigned char)(b)));\
                   asm ("bpl . + 9"); \
                   asm ("nega");\
                   asm ("tstb");\
                   asm ("bpl . + 9"); \
                   asm ("negb");\
                   asm ("bpl . + 17");\
                   asm ("tstb"); \
                   asm ("bpl . + 14"); \
                   asm ("negb"); \
                   asm ("mul"); \
                   asm ("std ___D");\
                   asm ("clra");\
                   asm ("clrb");\
                   asm ("subd ___D");\
                   asm ("bra . + 3");\
                   asm ("mul");  \
                   asm ("aslb");\
                   asm ("rola");\
                   asm ("aslb");\
                   asm ("rola");\
                   asm ("tfr a,b":"=B" ((unsigned char) __B)::"a", "b", "d");

// use this kind of init like:
//
// INIT_XYZ_A(_1_0_0);
// INIT_XYZ_A(_0_1_0);
// INIT_XYZ_A(_0_0_1);
//
// do the calculation here!
// for non special cases that is... (none left!)
// is used by makros below
#define CALC_XYZ(x,y,z) \
{\
 \
 if (z == 0)\
 {\
  if (y==0)\
  {\
   z = 0;\
   y = 0;\
  }\
  else\
  if (y == 1)\
  {\
   z = cosx;\
   y = -sinx;\
  }\
  else\
  {\
   z = -cosx;\
   y = sinx;\
  }\
 }\
 else if (z == 1)\
 {\
  if (y == 0)\
  {\
   z = sinx;\
   y = cosx;\
  }\
  else if (y == 1)\
  {\
   z = sinx + cosx;\
   y = cosx - sinx;\
  }\
  else\
  {\
   z = sinx - cosx;\
   y = cosx  + sinx;\
  }\
 }\
 else\
 {\
  if (y == 0)\
  {\
   z = -sinx;\
   y = -cosx;\
  }\
  else if (y == 1)\
  {\
   z = -sinx + cosx;\
   y = -cosx - sinx;\
  }\
  else\
  {\
   z = -sinx - cosx;\
   y = -cosx + sinx;\
  }\
 }\
 \
 MUL_B(z,siny);\
 xl = ((signed char) __B);\
 if (x == 1)\
 {\
  xl += cosy;\
 }\
 else if (x != 0)\
 {\
  xl -= cosy;\
 }\
 x= xl;\
 \
 \
 MUL_B(y,cosz);\
 yl = ((signed char) __B);\
 MUL_B(x,sinz);\
 yl += ((signed char) __B);\
 MUL_B(x,cosz);\
 x = ((signed char) __B);\
 MUL_B(y,sinz);\
 x -= ((signed char) __B);\
 y = yl;\
 \
}

#define INIT_INIT(address)\
 x = *(&allDirs[0] + (address - &allDirs_calc[0]));\
 y = *(&allDirs[0] + (address - &allDirs_calc[0])+1);\
 z = *(&allDirs[0] + (address - &allDirs_calc[0])+2);

#define EXIT_INIT(address)\
 *(&allDirs_calc[0] + (address - _0_0_0)) = x;\
 *(&allDirs_calc[0] + (address - _0_0_0)+1) = y;

#define EXIT_INIT_I(address)\
 *(&allDirs_calc[0] + (address - _0_0_0) + INVERS_OFFSET) = -x;\
 *(&allDirs_calc[0] + (address - _0_0_0) + INVERS_OFFSET+1) = -y;

#define INIT_XYZ(address)\
{\
 INIT_INIT(address);\
 CALC_XYZ(x,y,z);\
 EXIT_INIT(address);\
}

#define INIT_XYZ_I(address)\
{\
 INIT_INIT(address);\
 CALC_XYZ(x,y,z);\
 EXIT_INIT_I(address);\
}

#define INIT_XYZ_A(address)\
{\
 INIT_INIT(address);\
 CALC_XYZ(x,y,z);\
 EXIT_INIT(address);\
 EXIT_INIT_I(address);\
}

void yz_init()
{
}

void z_init()
{
}

// builds from a Line3d figure a 'usual' Vectrex Draw_VP list
void build_display(struct Line3d *figure)
{
 register unsigned char* pointer;
 register struct Line3d* linePointer;
 pointer = pvlist;
 linePointer = figure;
 *(pointer) = linePointer->pattern;
 while (*(pointer) != 1)
 {
  *(pointer+1) = linePointer->moveTo[1];// y
  *(pointer+2) = linePointer->moveTo[0];// x
  pointer+=3;
  linePointer++;
  *(pointer) = linePointer->pattern;
 }
}
void draw_3d(struct Line3d *figure)
{
 build_display(figure);
 draw_vector_list_pattern(pvlist);
}

*/
