/*
 * Include some standard vectrex functions first!
 */

#include <vectrex.h>
#include <ymsound.h>

/* all the 3d stuff */
#include "3d.h"

/* NON CONSTANT VARIABLES NOT ALLOWED TO BE INITIALIZED */

/*
 * Some defines, Maximal brightness is $7f, highest not set!
 * Max Scale would be $ff, well here we take only $f0!
 */
#define MAX_BRIGHTNESS (0x7f)
#define BASE_STRENGTH 50
#define SHOT_SCALE 120                  /* for positioning the dots on the screen */
#define SHOTS 34                        /* how many dots at one time? */
#define DOT_BRIGHTNESS 5                /* dot dwell time... */

/* These are the defines for the music */
/* they are globale symbals generated by ym_vpack.exe */
extern const void GMS_data;
extern const void PABLO5_data;
extern const void BITTLOAD_data;
extern const void SC6_data;
extern const void BLAST1_data;
extern const void ELITE1_data;
extern const void BADDIE_data;
extern const void JH3_data;

/*
 * Inlined static... insures this is 'really' inlined completely...
 *
 * oops... a small one :-)
 *
 */
static inline void start_one_vectrex_round(void)
{
  ym_sound();
  wait_recal();
}

/* well these are not actuall includes */
/* but having them in seperate files seems 'cleaner' somehow */
#include "intro.c"
#include "cube.c"
#include "pyramid.c"
#include "bird.c"
#include "last.c"
#include "elite.c"
#include "rip.c"

/*
 * Main ... what else to say?
 */
unsigned short demoState;

int main(void)
{
  unsigned char current_shot_kind;
  angle_x = 0;
  angle_y = 0;
  angle_z = 0;
  demoState = 0;

  current_shot_kind = 0;

  while (true)
  {
    start_one_vectrex_round();  /* start vectrex round */
    intensity(MAX_BRIGHTNESS);  /* set intensity of vector beam... */

    switch (demoState)
    {
     case 0:
     {
      intro();
      demoState++;
      break;
     }
     case 1:
     {
      intro_cube();
      demoState++;
      break;
     }
     case 2:
     {
      cube_demo();
      demoState++;
      break;
     }
     case 3:
     {
      intro_bird();
      demoState++;
      break;
     }
     case 4:
     {
      bird_demo();
      demoState++;
      break;
     }
     case 5:
     {
      intro_sbird();
      demoState++;
      break;
     }

     case 6:
     {
      birds_demo();
      demoState++;
      break;
     }
     case 7:
     {
      intro_pyramid();
      demoState++;
      break;
     }
     case 8:
     {
      pyramid_demo();
      demoState++;
      break;
     }
     case 9:
     {
      intro_p2yramid();
      demoState++;
      break;
     }
     case 10:
     {
      p2yramid_demo();
      demoState++;
      break;
     }
     case 11:
     {
      intro_last();
      demoState++;
      break;
     }
     case 12:
     {
      last_demo();
      demoState++;
      break;
     }
     case 13:
     {
      intro_elite();
      demoState++;
      break;
     }
     case 14:
     {
      elite_demo();
      demoState++;
      break;
     }
     case 15:
     {
      intro_rip();
      demoState++;
      break;
     }

     case 16:
     {
      rip_demo();
      demoState++;
      break;
     }
     default:
     {
      demoState = 0;
      break;
     }
    }
    read_buttons();             /* read buttons, if pressed... finish here */
    while (joystick1_button1) read_buttons();
 }
}
/* END OF FILE */