;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;                         draw_cl
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; taken from "RUMCHIP.ASM" file, found somewhere...
; corrected, commented and renamed labels (by Malban)
;
; subroutine for printing curved lines
; expects pointer to update table in U
; expects DP pointing to $D0
;
; update table has following format:
;                   DB  Y_update, VIDEO_enable, X_update, Xupdate,... ,0
;                   DB  Y_update, VIDEO_enable, X_update, Xupdate,... ,0
;                   DB  ...
;                   DB  0       ; 0 end of update list
;
; VIDEO_enable is a SHIFTREG poke
; this means $ff             == solid line
;            $00             == invisible line
;            everything else == somewhat dotted line
;
; Note: smoothest curved lines with X slopes
;       Y slopes tend to have some edges (since the integration must be
;       stopped for Y updates)
;

Vport_b=0HD000
Vport_a=HD001
Vshift_reg=0HD00A
Vaux_cntl=0HD00B

        .area __code
        .globl  _draw_cl_
_draw_cl_:
                 ldd     #0H1881           ; load D with VIA pokes
                 stb     Vport_b           ; poke $81 to port B
                                           ; disable MUX
                                           ; disable ~RAMP
                 sta     Vaux_cntl         ; poke $18 to AUX
                                           ; shift mode 4
                                           ; PB7 not timer controlled
                                           ; PB7 is ~RAMP
                 bra     next_update_round ; jump to entry of loop
xu_init_loop:
                 stb     Vport_b           ; MUX disable, ~RAMP enable
                 sta     Vshift_reg        ; poke the enable byte (A) found to
                                           ; shift, that enables/disables ~BLANK
xu_loop:
                 lda     ,u+               ; load next X_update value
                 beq     finnish_x_update  ; if zero, we are done with this
                                           ; X_update
                 sta     Vport_a           ; otherwise put the found value to
                                           ; DAC and thus to X integration
                 bra     xu_loop           ; go on, look if another X_update
                                           ; value is there...
finnish_x_update:
                 ldb     #0H81             ; load value for ramp off, MUX off
                 stb     Vport_b           ; poke $81, ramp off, MUX off

                 nop                       ; these NOP's seem to be neccessary
                 nop                       ; since the delay between VIA and
                 nop                       ; integration hardware
                 nop                       ; otherwise, there is a space
                                           ; between Y_updates...    Chris

                 sta     Vshift_reg        ; A == %00000000
next_update_round:
                 lda     ,u+               ; load next Y_update
                 beq     done_curved_line  ; go to done if 0
                 sta     Vport_a           ; poke to DAC
                 decb                      ; B now $80
                 stb     Vport_b           ; enable MUX, that means put
                                           ; DAC to Y integrator S/H
                 ldd     ,u++              ; A=VIDEO_enable, B=X_update
                 inc     Vport_b           ; MUX off, only X on DAC now
                 stb     Vport_a           ; store B (X_update) to DAC
                 ldb     #0H01             ; load poke for MUX disable,
                                           ; ~RAMP enable
                 bra     xu_init_loop      ; goto x update loop
done_curved_line:
                 lda     #0H98             ; load AUX setting
                 sta     Vaux_cntl         ; restore usual AUX setting
                                           ; (enable PB7 timer, SHIFT mode 4)
                 rts                       ; and out of here
